import { ElementAttrs, SVGElementTag } from "./types";

export default class ElementBuilder {

  private _elem: SVGElement;

  constructor(label: SVGElementTag) {
    this._elem = createElement(label);
  }

  setAttrs(attrs: ElementAttrs) {
    for (let [key, val] of Object.entries(attrs)) {
      this.attr(key, val);
    }
  }
  attr(name: string, val?: any): this | string {
    if (val) {
      setAttribute(this._elem, name, val);
      return this;
    }
    return getAttribute(this._elem, name) || '';
  }

  style(name: string, value: string) {
    this._elem.style.setProperty(name, value);
    return this;
  }

  get element() {
    return this._elem;
  }

  append(...elems: ElementBuilder[]): this {
    this._elem.append(...elems.map(elem => elem._elem));
    return this;
  }

  empty() {
    this._elem.innerHTML = '';
    return this;
  }

  static create(label: SVGElementTag, attrs?: ElementAttrs) {
    const instance = new ElementBuilder(label);
    if (attrs) {
      instance.setAttrs(attrs);
    }
    return instance;
  }
}

const SVG_NAMESPACE = 'http://www.w3.org/2000/svg';

function createElement(tag: SVGElementTag) {
  return document.createElementNS(SVG_NAMESPACE, tag);
}

function setAttribute(element: SVGElement, name: string, value: any) {
  if (name === 'xlink:href') {
    element.setAttributeNS(SVG_NAMESPACE, name, value);
  }
  element.setAttribute(name, value);
}

function getAttribute(element: SVGElement, name: string) {
  return element.getAttribute(name);
}