export type TickCallback = (percentageComplete: number) => void;

export class Tick {
  private _tickCallback: TickCallback;
  private _startTime: number = 0;
  private _duration: number = 0;
  private _stopped = false;
  private _pending = false;

  constructor(cb: TickCallback) {
    this._tickCallback = cb;
  }

  start(duration: number) {
    if (this._pending) return;

    this._duration = duration;
    this._startTime = Date.now();
    this.tickLoop();
  }

  private tickLoop() {
    this._pending = true;
    const tick = () => {
      let percentageComplete = (Date.now() - this._startTime) / this._duration;
      let over = false;

      if (percentageComplete > 1) {
        percentageComplete = 1;
        over = true;
      };
      if (!this._stopped) {
        this._tickCallback(percentageComplete);
      }
      
      if (!over && !this._stopped) {
        requestAnimationFrame(tick);
      } else {
        this._pending = false;
      }
    }

    requestAnimationFrame(tick);
  }

  stop() {
    this._stopped = true;
  }

}