import { Ease, easeLinear } from "./ease";
import { Tick } from './tick';
import { interpolateNumber, interpolateNumberArray } from './interpolate';
import { compose } from "./compose";

export type TransitionOption<T> = {
  ease?: Ease;
  duration: number;
  from: T;
  to: T;
};

export function transition(
  option: TransitionOption<number | number[]>,
  cb:(current: unknown) => void
) {
  const {
    from,
    to,
    ease = easeLinear,
    duration
  } = option ?? {};

  if (from instanceof Array && !(to instanceof Array)) {
    throw 'Field from and to must have the same type.'
  }

  let interpolateFunc = from instanceof Array
    ? interpolateNumberArray(from, to as number[])
    : interpolateNumber(from, to as number);

  const tick = new Tick(t => compose(cb, interpolateFunc, ease)(t));
  tick.start(duration);

  return () => tick.stop();
}